
#ifndef _STRUCTS_H_
#define _STRUCTS_H_

struct _TreeNodeList;

// If space is an issue, we can eliminated some of the
// fields in the TreeNode structure.
//
// - "Parent" can be removed but requires a few changes to
//   the ReadInTree() procedure in utils.c.
// - "SamplesRootedHere" can be removed by recalculating the
//   number of samples during the sampling phase using
//   numValidLDescendents and numValidRDescendents.
// - "Descendents" can be obtained by traversing the subtree
//   each time (although this clearly increases the runtime).

typedef struct _TreeNode {

  double depth;
  double SamplesRootedHere;
  unsigned long numValidLDescendents;
  unsigned long numValidRDescendents;

  char * Name;

  struct _TreeNode * Parent;
  struct _TreeNode * LChild;
  struct _TreeNode * RChild;

  // List of leaf descendents of this node
  struct _TreeNodeList * Descendents;
  
} TreeNode;

typedef struct _TreeNodeList {
  struct _TreeNode * Node;
  struct _TreeNodeList * Next;
} TreeNodeList;

typedef struct _InducedSubtree {

  struct _TreeNode * Node;
  struct _InducedSubtree * Parent;
  struct _InducedSubtree * LChild;
  struct _InducedSubtree * RChild;

} InducedSubtree;

#endif
